import sys
import lib6502
from libsid import SIDFile, makeDriver

"""
LTL operators:
	X, next, a X b means b happens right after a
	G, globally, G a means a is always true
	F, eventually, F a means a becomes true at some point
	U, until, a U b means a is true at least until b becomes true
	R, release, a R b means b is true until (and including) the point where
		a becomes true. b is true forever if a never becomes true.
"""

class Error(Exception):
	pass

class SIDInfo(object):
	def __init__(self):
		self.periodicPlaybackRate = None

class CheckSID(object):
	def __init__(self, sid):
		self.sid = sid
	
	def check(self):
		# Is this a PSID or an RSID?
		if self.sid.isPSID:
			self.checkPSID()
		else:
			raise Error("Non-PSID not yet supported.")
	
	def checkPSID(self):
		if self.sid.songSpeed != 0:
			self.checkTimer1Write()

	def printInfo(self):
		print "%s, by %s:" % (self.sid.songName, self.sid.songComposer)

	def checkTimer1Write(self):
		system = self.getSystemWithData()
		system.setExitOnBrk()

		system.run()
		print "Executed?"
	
	def getSystemWithData(self):
		system = lib6502.System()

		# Create a SID player driver -- song number 0, only run init routine
		driver = makeDriver(self.sid.initAddr, self.sid.playAddr, 0,
				brkAfterInit = True)

		pages = self.sid.pages.copy()
		pages[0xff00] = driver

		# Load the SID into the system.
		for pageNum, pageData in pages.items():
			for idx in range(len(pageData)):
				addr = pageNum | idx
				system.writeb(addr, ord(pageData[idx]))

		# Re-reset the system to grab the new start vector from the
		# just-uploaded driver code.

		system.reset()
		return system

def main(argv):
	sid = SIDFile()
	sid.load(argv[1], printSIDInfo = False)

	checkSID = CheckSID(sid)
	checkSID.check()
	checkSID.printInfo()

if __name__ == '__main__':
	main(sys.argv)

