import socket
import serial
import struct
import time
import sys
import os

BPS = 115200

class Device(object):
	def __init__(self, deviceName):
		self.con = None
		self.connect(deviceName)
		# Wait for the chip to say hi.
		#self.transact('\r\n')

	def transact(self, dataIn):
		#print "transact...", dataIn
		self.con.write(dataIn)
		dataOut = []
		while 1:
			dataOut.append(self.con.read(1))
			#print "data", dataOut[-1], hex(ord(dataOut[-1]))
			if dataOut[-1] == '>' and dataOut[-2] == '\n' and dataOut[-3] == '\r':
				# Done here..
				return ''.join(dataOut)

	def connect(self, deviceName):
		if self.con is None:
			self.con = serial.Serial(port = deviceName, baudrate = BPS)
			#print "sleep"
			time.sleep(1)
			#print "okay..."
	
	def disconnect(self):
		self.con.close()

class SIDServer(object):
	def __init__(self, serialDevice, port):
		self.port = port
		try:
			self.device = Device(serialDevice)
		except serial.SerialException:
			print "Couldn't initialise serial device."
			self.device = None
			raise

	def spawn(self):
		self.ssock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		self.ssock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
		self.ssock.bind(('localhost', self.port))
		self.ssock.listen(1)
		self._daemonise()
		self._serve()
	
	def _serve(self):
		while 1:
			sock, addr = self.ssock.accept()
			while 1:
				cmdLength = sock.recv(2)
				if not cmdLength:
					# Socket closed.
					break
				cmdLength = struct.unpack('!H', cmdLength)[0]
				if cmdLength == 0:
					sock.close()
					break
				else:
					cmd = sock.recv(cmdLength)
					result = self.device.transact(cmd)
					sock.send(struct.pack('!H', len(result)))
					sock.send(result)

	def _daemonise(self):
		# do the UNIX double-fork magic, see Stevens' "Advanced 
		# Programming in the UNIX Environment" for details (ISBN 0201563177)
		try: 
			pid = os.fork() 
			if pid > 0:
				# exit first parent
				sys.exit(0) 
		except OSError, e: 
			print >>sys.stderr, "fork #1 failed: %d (%s)" % (e.errno, e.strerror) 
			sys.exit(1)

		# decouple from parent environment
		os.chdir("/") 
		os.setsid() 
		os.umask(0) 

		# do second fork
		try: 
			pid = os.fork() 
			if pid > 0:
				# exit from second parent, print eventual PID before
				print "Daemon PID %d" % pid 
				sys.exit(0) 
		except OSError, e: 
			print >>sys.stderr, "fork #2 failed: %d (%s)" % (e.errno, e.strerror) 
			sys.exit(1) 

class SIDServerClient(object):
	def transact(self, dataIn):
		pass

